char dfreebytes_c_rcs_id [] = 
	"$Id: dfreebytes.c,v 1.2 1995/05/19 19:35:00 hjp Exp $";
/*
 *	dfreebytes
 *
 *	report free bytes on given drive (given by the directory the FS
 *	is mounted on.
 */



#if __minix
/*
 *	Parses output from df. Df is supposed to produce a table like the 
 *	following:
 *
 *            1k-Blocks    free    used    %   Files  used%   Mounted on
 * /dev/hd3        1144      74    1070  94%     480    58%   /
 * /dev/hd4       25064     561   24503  98%    8362    43%   /usr
 * /dev/hd2       39312    2994   36318  93%   13112    32%   /usr/local
 * /dev/hd8       37128     868   36260  98%   12384    19%   /home
 * /dev/ram        1024     997      27   3%     352     1%   /tmp
 * /dev/fd1        1440       0    1440 100%     480     5%   /b
 *
 *	$Log: dfreebytes.c,v $
 *	Revision 1.2  1995/05/19 19:35:00  hjp
 *	Quick'n'dirty port to linux.
 *	Compiles but no guarantees that it works.
 *
 * Revision 1.1.1.1  1995/05/19  18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.3  1993/06/25  15:07:51  hjp
 * adapted to output of vmd df
 *
 * Revision 1.2  1992/05/30  22:43:56  hjp
 * fixed parsing of header lines
 *
 */


#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <blocksize.h>
#include <unistd.h>

#include <ant/io.h>
#include <ant/disk.h>
#include <ant/string.h>

/* nowhere else declared */
FILE *popen (const char *command, const char *mode);
int pclose (FILE *pp);

extern char *cmnd;

	long
dfreebytes (
	char const * drive
){

	char	buf [128];
	char	devstr [16],
		inotstr [16],
		inoustr [16],
		inofstr [16],
		inopstr [16],
		blktstr [16],
		blkustr [16],
		blkfstr [16],
		blkpstr [16],
		mntstr [16],
		verstr [16],
		prmstr [16];
	int	inotint,
		inouint,
		inofint,
		inopint,
		blktint,
		blkuint,
		blkfint,
		blkpint;

        FILE	*pp;

	if ((pp = popen ("df", "r")) == NULL) {
		eprintf ("%s: dfreebytes: cannot execute df: %s\n",
			cmnd, strerror (errno));
		exit (1);
	}
	if (fgets (buf, sizeof (buf), pp) == NULL || 
	    sscanf (buf, " %s%s%s%s %s%s %s",
		    blktstr, blkfstr, blkustr, blkpstr,
		    inoustr, inopstr,
		    mntstr) != 7 ||
	    !STREQ (inoustr, "Files") ||
	    !STREQ (inopstr, "used%") ||
	    !STREQ (blktstr, "1k-Blocks") ||
	    !STREQ (blkustr, "used") ||
	    !STREQ (blkfstr, "free") ||
	    !STREQ (blkpstr, "%") ||
	    !STREQ (mntstr, "Mounted")
	   ) {
		eprintf ("%s: dfreebytes: malformed or missing first"
			 " header line in output from df\n", cmnd);
		eprintf ("buf = `%s'\n", buf);
		eprintf ("inoustr = `%s' ", inoustr);
		eprintf ("inopstr = `%s' ", inopstr);
		eprintf ("blktstr = `%s' ", blktstr);
		eprintf ("blkustr = `%s' ", blkustr);
		eprintf ("blkfstr = `%s' ", blkfstr);
		eprintf ("blkpstr = `%s' ", blkpstr);
		eprintf ("mntstr = `%s'\n", mntstr);
	 	exit (1);
	}
	while (fgets (buf, sizeof (buf), pp)) {
		if (sscanf (buf, "%s %d%d%d%d%% %d%d%% %s",
			    devstr,
			    &blktint, &blkfint, &blkuint, &blkpint,
			    &inouint, &inopint,
			    mntstr) != 8) {
			eprintf ("%s: dfreebytes: malformed"
				 " body line in output from df\n", cmnd);
			eprintf ("buf = `%s'\n", buf);
			eprintf ("devstr = `%s' ", devstr);
			eprintf ("inouint = %d ", inouint);
			eprintf ("inopint = %d ", inopint);
			eprintf ("blktint = %d ", blktint);
			eprintf ("blkuint = %d ", blkuint);
			eprintf ("blkfint = %d ", blkfint);
			eprintf ("blkpint = %d ", blkpint);
			eprintf ("mntstr = `%s'\n", mntstr);
			exit (1);
		}
		if (STREQ (mntstr, drive)) {
			pclose (pp);
			return blkfint * BLOCK_SIZE;
		}
	}
	pclose (pp);

        return -1;
}

#endif




#if __linux
/* Linux has a statfs call, so we can use that (a lot better than parsing
 * output of df, should implement that in Minix, too
 */
#include <sys/vfs.h>

#define DEBUG 1
#if DEBUG 
#include <stdio.h>
#define printval(v) fprintf (stderr, #v ": %ld\n", (long)v);
#else
#define printval(v)
#endif

	long
dfreebytes (
	char const * drive
){
	struct statfs sfb;

	if (statfs (drive, &sfb) == -1) return -1;

	printval (sfb.f_bsize);
	printval (sfb.f_blocks);
	printval (sfb.f_bfree);
	printval (sfb.f_bavail);

	return  sfb.f_bsize * sfb.f_bavail;
}

#endif
