char efdregister_c_rcs_id[] =
	"$Id: efdregister.c,v 1.1 1997/01/23 20:45:43 hjp Exp $";


#include <assert.h>
#include <limits.h>
#include <stdlib.h>
#include <ant/io.h>
#include <ant/string.h>

char *efdregister(int fd, const char *filename) {
	/* register a file name with a file descriptor. This is very handy 
	 * for error messages, because you don't have to pass
	 * both the file descriptor and the file name around.
	 *
	 * efdregister is used both to register the file name and
	 * to query it: If filename is NULL, the file name is not
	 * set. In any case the previously registered file name
	 * is returned (which may be null and is probably of dubious
	 * value unless efdregister has been called previously).
	 *
	 * eopen calls efdregister to register the newly opened 
	 * file. Some other functions may use it to get a file
	 * name for error messages.
	 */

	static struct {
		char *filename;
	} table [OPEN_MAX];

	char *prevfilename = table[fd].filename;

	assert(fd >= 0 && fd < OPEN_MAX);

	if (filename) {
		table[fd].filename = strdup(filename);
                /* Argh - we have a memory leak here:
                 * We cannot free the previous filename because me
                 * must return it. The caller cannot free it either,
                 * unless it knows it will never be used again.
                 * The best compromise could be to free the filename
                 * in eclose.  That way the filename has at least a
                 * defined lifetime.
                 */
	}

	return prevfilename;
}
