char efregister_c_rcs_id[] =
	"$Id: efregister.c,v 1.2 1996/06/11 21:57:47 hjp Exp $";


#include <assert.h>
#include <limits.h>
#include <stdlib.h>
#include <ant/io.h>
#include <ant/string.h>

#ifndef OPEN_MAX
#define OPEN_MAX 256
#endif

char *efregister(FILE *fp, const char *filename) {
	/* register a file name with a stream. This is very handy 
	 * for error messages, because you don't have to pass
	 * both the stream and the file name around.
	 *
	 * efregister is used both to register the file name and
	 * to query it: If filename is NULL, the file name is not
	 * set. In any case the previously registered file name
	 * is returned (which may be null and is probably of dubious
	 * value unless efregister has been called previously).
	 *
	 * efopen calls efregister to register the newly opened 
	 * file. Some other functions may use it to get a file
	 * name for error messages.
	 */

	static struct {
		FILE *fp;
		char *filename;
	} table [OPEN_MAX];

	int fd = fileno(fp);
	char *prevfilename;

	assert(fd >= 0 && fd < OPEN_MAX);

	if (filename) {
		if (fp == table[fd].fp) {
			prevfilename = table[fd].filename;
		} else {
			table[fd].fp = fp;
			if (table[fd].filename) {
				free(table[fd].filename);
			}
			prevfilename = NULL;
		}
		table[fd].filename = strdup(filename);
	} else {
		if (fp == table[fd].fp) {
			prevfilename = table[fd].filename;
		} else {
			prevfilename = NULL;
		}
	}

	return prevfilename;
}
