char get_label_rcs_id [] = 
	"$Id: getlabel.c,v 1.2 1996/06/11 21:57:48 hjp Exp $";
/*
 *	getlabel
 *
 *	returns the label of the disk in the drive given. Returns a null
 *	string if the disk has no label (I think this gives better result
 *	than returning a NULL pointer).
 *
 *	Minix disks do not have a label the way DOS disks have. I am
 *	using the first file starting with a blank and having zero
 *	length as label.
 *
 *	The space for the label is malloced and must be freed if no
 *	longer needed.
 *
 *	$Log: getlabel.c,v $
 *	Revision 1.2  1996/06/11 21:57:48  hjp
 *	Changes to compile on HP-UX (debugging comes later).
 *
 * Revision 1.1.1.1  1995/05/19  18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.3  1993/06/25  15:09:44  hjp
 * added debugging
 *
 * Revision 1.2  1992/05/30  22:50:07  hjp
 * adapted to MINIX
 *
 *	1.0	91-08-23
 */

#include <errno.h>
#include <limits.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>

#include <ant/disk.h>
#include <ant/string.h>

/*#define DEBUG 1 */

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

	char *
getlabel (
	char const * drive
){
        DIR		*dp;
        struct dirent	*de;
        struct stat	sb;
        char		filename [PATH_MAX];

	if ((dp = opendir (drive)) == NULL) {
#ifdef DEBUG
		printf ("getlabel: cannot open `%s': %s\n",
			drive, strerror (errno));
#endif
		return "";
	}

	while (de = readdir (dp)) {
#ifdef DEBUG
		printf ("d_name = `%s'\n", de->d_name);
#endif
		if (*de->d_name == ' ') {
			nstrncpy (
				nstrncpy (
					nstrncpy (
						filename,
						drive,
						filename + PATH_MAX
					), "/",
					filename + PATH_MAX
				), de->d_name,
				filename + PATH_MAX);
#ifdef DEBUG
			printf ("filename = `%s'\n", filename);
#endif
			if (stat (filename, &sb) == 0 && sb.st_size == 0) {
				closedir (dp);
				return strdup (de->d_name + 1);
			}
		}
	}
#ifdef DEBUG
	printf ("getlabel: closedir\n");
#endif

	closedir (dp);
        return "";
}
