#include <assert.h>
#include <time.h>
#include <ant/time.h>

long gettzoff(time_t when) {
        /* Return offset in seconds of local timezone relative to UTC
         * at time |when|.
         */
         
        struct tm utm, ltm;
        long off = 0;

        utm = *gmtime(&when);
        ltm = *localtime(&when);

        /* Check for wrapping at a day boundary. 
         * This can amount to at most one day, so we check only
         * if the date is earlier or later, not by how much
         * (This would require knowledge of leap years -
         * not too difficult and probably useful for a debugging
         * library).
         */
        if      (ltm.tm_year > utm.tm_year) off =  24 * 3600;
        else if (ltm.tm_year < utm.tm_year) off = -24 * 3600;
        else if (ltm.tm_yday > utm.tm_yday) off =  24 * 3600;
        else if (ltm.tm_yday < utm.tm_yday) off = -24 * 3600;

        /* Compute offset. Comparing seconds is probably unneccessary,
         * but comparing minutes is not - there are timezones with 
         * half-hour offsets
         */
        off += (ltm.tm_hour - utm.tm_hour) * 3600 
             + (ltm.tm_min  - ltm.tm_min ) * 60
             + (ltm.tm_sec  - ltm.tm_sec );

        /* Sanity check. 14 hour offsets do currently exist, but
         * everything more is either an error or a major change in
         * geographics.
         */
        assert (-14*3600 <= off && off <= 14*3600);
        return off;
}
