#ifndef I_ant_io_h /* { */
#define I_ant_io_h
#if INCLUDE_IDENT /* { */
static char ant_io_h_rcs_id [] = 
	"$Id: io.h,v 1.3 1997/01/23 20:45:45 hjp Exp $";
#endif /* } */
/*
 * $Id: io.h,v 1.3 1997/01/23 20:45:45 hjp Exp $
 *
 * I/O-related functions from ANT
 *
 * $Log: io.h,v $
 * Revision 1.3  1997/01/23 20:45:45  hjp
 * Merged changes in two CVS repositories (outch!).
 *
 * Revision 1.2  1996/06/11  21:20:48  hjp
 * Added efregister
 * Added Configuration for Linux-ELF.
 *
 * Revision 1.1.1.1  1995/05/19  18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.7  1993/06/28  22:06:06  hjp
 * added writeu{long,short}[bl]e
 *
 * Revision 1.6  1993/06/25  14:43:09  hjp
 * added getline
 * added rcs id
 *
 * Revision 1.5  1993/02/05  20:52:03  hjp
 * added readshort[lb]e
 *
 * Revision 1.4  1992/12/17  04:22:07  hjp
 * added readlong*
 *
 * Revision 1.3  1992/12/17  00:35:20  hjp
 * added getnextline
 *
 * Revision 1.2  1992/05/30  23:04:14  hjp
 * *** empty log message ***
 *
 * Revision 1.1  1992/05/30  18:11:03  hjp
 * Initial revision
 *
 */
#include <stdio.h>
#include <ant/bool.h>

extern void eprintf (const char *fmt, ...);
extern void efclose (FILE *fp);
extern FILE *efopen (const char *filename, const char *mode);
extern int eopen (const char *filename, int flags, ...);
extern char *efregister (FILE *fp, const char *filename);
extern char *efdregister (int, const char *filename);
extern bool getnextline (FILE *fp);

long readlongle (FILE *fp);
	/*
	 * Read a long value stored in little endian byte order
	 *
	 * CAUTION: EOF is a valid long value. You have to check
	 * for feof, unless you know the long read has to be nonnegative
	 */
long readlongbe (FILE *fp);
	/*
	 * Read a long value stored in big endian byte order
	 *
	 * CAUTION: EOF is a valid long value. You have to check
	 * for feof, unless you know the long read has to be nonnegative
	 */
int readshortle (FILE *fp);
	/*
	 * Read a short value stored in little endian byte order
	 *
	 * CAUTION: if sizeof (short) == sizeof (int) then EOF is a valid
	 * short value. You have to check for feof, unless you know the
	 * short read has to be nonnegative
	 */
int readshortbe (FILE *fp);
	/*
	 * Read a short value stored in big endian byte order
	 *
	 * CAUTION: if sizeof (short) == sizeof (int) then EOF is a valid
	 * short value. You have to check for feof, unless you know the
	 * short read has to be nonnegative
	 */
char *getline (FILE *fp);
	/*
	 * Purpose: read one line from file
	 * Algorithm:	characters are read from fp until EOF or '\n' is read
	 *		space is malloced as needed, the line terminator '\n' is
	 *		replaced by '\0'.
	 *		The string is reused on the next call to getline!
	 */
int writeulongle (unsigned long l, FILE *fp);
	/*
	 * Write an unsigned long value in little endian byte order
	 */
int writeulongbe (unsigned long l, FILE *fp);
	/*
	 * Write an unsigned long value in big endian byte order
	 */
int writeushortle (unsigned short l, FILE *fp);
	/*
	 * Write an unsigned short value in little endian byte order
	 */
int writeushortbe (unsigned short l, FILE *fp);
	/*
	 * Write an unsigned short value in big endian byte order
	 */
#endif
