char openport_c_rcs_id[] =
	"$Id: openport.c,v 1.1 1997/01/23 20:45:47 hjp Exp $";
/* openport -- Open a TCP connection to a port on the specified host.
 *
 * $Log: openport.c,v $
 * Revision 1.1  1997/01/23 20:45:47  hjp
 * Merged changes in two CVS repositories (outch!).
 *
 */
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <netdb.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include <ant/net.h>

int openport(
	const char *hostname,
	int default_port
){
	/* Open a TCP connection to a port on the specified host.
	 * The host can be given as a DNS name or in dotted quad
	 * notation. An optional port number can be appended after
	 * a colon. If no port number is given, default_port is 
	 * used.
	 *
	 * Return value: A file descriptor, or -1 on error
	 */
	int port;
	char hname[1024];
	struct sockaddr_in serv_addr;
	char *p;
	struct hostent *she;
	unsigned int dq[4];
	char dummy;
	int sd;

	if ((p = strchr(hostname, ':'))) {
		port = atoi(p + 1);
		assert(p - hostname < sizeof(hname));
		memcpy(hname, hostname, p - hostname);
		hname[p - hostname] = '\0';
		hostname = hname;
	} else {
		port = default_port;
	}
	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	if (sscanf(hostname, "%u.%u.%u.%u%c",
		&dq[0], &dq[1], &dq[2], &dq[3], &dummy) == 4) {
		assert(sizeof(serv_addr.sin_addr) == 4);
		((unsigned char *)&serv_addr.sin_addr)[0] = dq[0];
		((unsigned char *)&serv_addr.sin_addr)[1] = dq[1];
		((unsigned char *)&serv_addr.sin_addr)[2] = dq[2];
		((unsigned char *)&serv_addr.sin_addr)[3] = dq[3];
	} else {
		she = gethostbyname(hostname);

		if (!she) {
                        #if 0
			fprintf(stderr, "%s: cannot lookup host %s:%s",
				cmnd, hostname, strerror(errno));
                        #endif
			return -1;
		}

		assert(sizeof(serv_addr.sin_addr) == she->h_length);
		memcpy(&serv_addr.sin_addr, she->h_addr, she->h_length);
	}
	serv_addr.sin_port = htons (port);

	if ( ( sd = socket ( AF_INET, SOCK_STREAM, 0 ) ) < 0 ) {
                #if 0
		fprintf(stderr, "%s: can't open stream socket:%s",
			cmnd, strerror(errno));
                #endif
		return -1;
	}

	if ( connect ( sd, (struct sockaddr *) &serv_addr,
			sizeof ( serv_addr) ) < 0 ) {
                #if 0
		fprintf(stderr, "%s: can't connect to server: %s",
			cmnd, strerror(errno) );
                #endif
		return -1;
	}
	return sd;
}
