char rdbcchdr_c_rcs_id [] = 
	"$Id: rdbcchdr.c,v 1.2 1997/03/31 20:45:06 hjp Exp $";
/*
 * read header of bcc object file
 *
 * Bcc object files differ from GNU object files in several ways:
 *	* They can contain several independent modules. Therefore
 *	  they should be treated like archives. But since bcc only
 *	  produces 1 module per file I am treating them as a single
 *	  object file and merge the file header and the header of
 *	  the first module. This function will fail if it encounters
 *	  a file with more than one module.
 *	* They do not contain a bss segment. Instead, objects which 
 *	  would normally be in a bss segment are listed as imported
 *	  with a size != 0. Therefore this function has to read the
 *	  symbol table to compute the size of the bss segment. After
 *	  doing this, the function seeks back to the end of the header.
 *	* Finally, the text segment does not immediately follow the
 *	  header. The function sets the textoff field to the start
 *	  of the text segment, so that the program can seek to the 
 *	  text segment.
 *
 * $Log: rdbcchdr.c,v $
 * Revision 1.2  1997/03/31 20:45:06  hjp
 * ecalloc
 *
 * Revision 1.1.1.1  1995/05/19 18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.2  1993/02/05  21:20:05  hjp
 * added rcs id and comment
 *
 */
#include <assert.h>
#include <stdlib.h>

#include <ant/alloc.h>
#include <ant/io.h>
#include <ant/readheader.h>

#define SZ_MASK		0xC000
#define SZ_SHIFT	14
#define N_MASK		0x0100
#define E_MASK		0x0080
#define I_MASK		0x0040
#define R_MASK		0x0020
#define A_MASK		0x0010
#define SEGM_MASK	0x000F

#define getshort	readshortle
#define getlong		readlongle



exec_headerT *read_bcc_header (FILE *fp) {
	unsigned		count;
	unsigned long		textoff, textsize, segmaxsize, segmaxdesc;
	unsigned char		checksum, class, revision;
	int			i, j;
	unsigned short		nsymbol, stringsize, flags;
	unsigned long		offset;
	static exec_headerT	header;
	fpos_t			after_header;

	header.type = BCC_OBJ;
	count = getshort (fp);
	assert (count == 1);	/* can only read one module per file */
	checksum = getc (fp);

	for (i = 0; i < count; i ++) {
		textoff = getlong (fp);
		header.textoff = ftell (fp) + textoff - 6;
			/* 6 == magic + textoff */
		textsize = getlong (fp);
		stringsize = getshort (fp);
		class = getc (fp);
		revision = getc (fp);
		segmaxsize = getlong (fp);
		assert (segmaxsize == 0x55555555);
		segmaxdesc = getlong (fp);
		for (i = 0; i < 16; i ++) {
			switch ((segmaxdesc >> (30 - i * 2)) & 3) {
			case 3:
				offset = getlong (fp);
				break;
			case 2:
				offset = getshort (fp);
				break;
			case 1: 
				offset = getc (fp);
				break;
			case 0:
				offset = 0;
				break;
			default:
				assert (0);
			}
			if (i == 0) header.text = offset;
			if (i == 3) header.data = offset;
			assert (offset == 0 || i == 0 || i == 3);
			/* only text and data segments are allowed */
		}
		assert (header.text + header.data == textsize);
		/* There is no bss segment in a bcc object file.
		 * symbols which will end up in the bss segment
		 * after linking can be recognized by having both 
		 * the import flag set and an offset (which really
		 * is their length, not their offset
		 *
		 * So we read the symbol table to sum up the length
		 * of such objects and call it bss
		 */
		fgetpos (fp, &after_header);

		header.bss = 0;
		nsymbol = getshort (fp);
		for (j = 0; j < nsymbol; j ++) {
			getshort (fp);
			flags = getshort (fp);
			switch ((flags & SZ_MASK) >> SZ_SHIFT) {
			case 3:
				offset = getlong (fp);
				break;
			case 2:
				offset = getshort (fp);
				break;
			case 1: 
				offset = getc (fp);
				break;
			case 0:
				offset = 0;
				break;
			default:
				assert (0);
			}
			if ((flags & I_MASK) && offset != 0) {
				header.bss += offset;
			}
		}
		header.total = header.text + header.data + header.bss;
		/* now set file position back to end of the header
		 * (Or maybe we should skip the string space, too,
		 * to find the text segment)
		 */
		fsetpos (fp, &after_header);

	}
	return &header;
}
