char readbin_c_rcs_id [] = 
	"$Id: readbin.c,v 1.1.1.1 1995/05/19 18:42:31 hjp Exp $";
/*
 * readbin -- functions to read binary files
 *
 * assumptions about data representation in files:
 *	Byte:	8 bits
 *	Short:	16 bits
 *	Long:	32 bits
 *	(i.e. the minimum sizes guarantueed by ANSI C)
 *
 * $Log: readbin.c,v $
 * Revision 1.1.1.1  1995/05/19 18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.2  1993/02/05  20:52:03  hjp
 * added readshort[lb]e
 *
 * Revision 1.1  1992/12/17  04:24:54  hjp
 * Initial revision
 *
 */

#include <stdio.h>
#include <ant/io.h>

long readlongle (FILE *fp) 
	/*
	 * Read a long value stored in little endian byte order
	 *
	 * CAUTION: EOF is a valid long value. You have to check
	 * for feof, unless you know the long read has to be nonnegative
	 */
{
	unsigned long v;
	v = getc (fp);
	v |= getc (fp) << 8;
	v |= getc (fp) << 16;
	v |= getc (fp) << 24;
	return feof (fp) ? EOF : v;
}


long readlongbe (FILE *fp) 
	/*
	 * Read a long value stored in big endian byte order
	 *
	 * CAUTION: EOF is a valid long value. You have to check
	 * for feof, unless you know the long read has to be nonnegative
	 */
{
	unsigned long v;
	v = getc (fp) << 24;
	v |= getc (fp) << 16;
	v |= getc (fp) << 8;
	v |= getc (fp);
	return feof (fp) ? EOF : v;
}


int readshortle (FILE *fp) 
	/*
	 * Read a short value stored in little endian byte order
	 *
	 * CAUTION: if sizeof (short) == sizeof (int) then EOF is a valid
	 * short value. You have to check for feof, unless you know the
	 * short read has to be nonnegative
	 */
{
	int v;
	v = getc (fp);
	v |= getc (fp) << 8;
	return feof (fp) ? EOF : v;
}


int readshortbe (FILE *fp) 
	/*
	 * Read a short value stored in big endian byte order
	 *
	 * CAUTION: if sizeof (short) == sizeof (int) then EOF is a valid
	 * short value. You have to check for feof, unless you know the
	 * short read has to be nonnegative
	 */
{
	int v;
	v = getc (fp) << 8;
	v |= getc (fp);
	return feof (fp) ? EOF : v;
}
