char readheader_c_rcs_id [] =
	"$Id: readheader.c,v 1.1.1.1 1995/05/19 18:42:31 hjp Exp $";
/*
 * read header of several types of object files. This should
 * make size, nm, etc. independent of object file format.
 * Changes and additions need only be made in one library 
 * function instead of several utilities
 */
#include <assert.h>
#include <stdio.h>

#include "readheader.h"

char *exec_type_strings [] = {
	"unknown",
	"Minix 8086 combined I&D executable",
	"Minix 80386 combined I&D executable",
	"Minix 68000 combined I&D executable",
	"Minix 8086 seperate I&D executable",
	"Minix 80386 seperate I&D executable",
	"Minix 68000 seperate I&D executable",
	"GNU big endian demand paged executable",
	"GNU little endian demand paged executable",
	"GNU big endian pure executable",
	"GNU little endian pure executable",
	"GNU big endian object file",
	"GNU little endian object file",
	"BCC object file",
	"Bootmon image",
};

exec_headerT *read_exec_header (FILE *fp) {
	int	c;

	assert (sizeof exec_type_strings / sizeof exec_type_strings [0] == NR_TYPES);
	switch (c = getc (fp)) {
	case MAGIC_MINIX0:
		switch (getc (fp)) {
		case MAGIC_MINIX1:
			return read_minix_header (fp);
		}
		break;
	case MAGIC_GNU_LE_PURE0:
		switch (getc (fp)) {
		case MAGIC_GNU_LE_PURE1:
			switch (getc (fp)) {
			case MAGIC_GNU_LE_PURE2:
				switch (getc (fp)) {
				case MAGIC_GNU_LE_PURE3:
					return read_gnu_header (
						fp, GNU_LE_PURE);
				}
			}
		}
		break;
	case MAGIC_GNU_LE_PAGE0:
		switch (getc (fp)) {
		case MAGIC_GNU_LE_PAGE1:
			switch (getc (fp)) {
			case MAGIC_GNU_LE_PAGE2:
				switch (getc (fp)) {
				case MAGIC_GNU_LE_PAGE3:
					return read_gnu_header (
						fp, GNU_LE_PAGE);
				}
			}
		}
		break;
	case MAGIC_GNU_LE_OBJ0:
		switch (getc (fp)) {
		case MAGIC_GNU_LE_OBJ1:
			switch (getc (fp)) {
			case MAGIC_GNU_LE_OBJ2:
				switch (getc (fp)) {
				case MAGIC_GNU_LE_OBJ3:
					return read_gnu_header (
						fp, GNU_LE_OBJ);
				}
			}
		}
		break;
	case MAGIC_GNU_BE0:
		switch (getc (fp)) {
		case MAGIC_GNU_BE1:
			switch (getc (fp)) {
			case MAGIC_GNU_BE2:
				switch (getc (fp)) {
				case MAGIC_GNU_BE_OBJ3:
					return read_gnu_header (
						fp, GNU_BE_OBJ);
				case MAGIC_GNU_BE_PAGE3:
					return read_gnu_header (
						fp, GNU_BE_PAGE);
				case MAGIC_GNU_BE_PURE3:
					return read_gnu_header (
						fp, GNU_BE_PURE);
				}
			}
		}
		break;

	case MAGIC_BCC0:
		switch (getc (fp)) {
		case MAGIC_BCC1:
			return read_bcc_header (fp);
			break;
		}
	default:
		/* Bootmon images don't have a magic number
		 * Just try to read it the header of the first
		 * entry and see if it makes sense
		 */
		ungetc (c, fp);
		return read_bootmon_header (fp);
	}
	return NULL;
}
