/*
 * $Id: string.h,v 1.1.1.1 1995/05/19 18:42:31 hjp Exp $
 *
 * String functions and Macros from ANT
 *
 * $Log: string.h,v $
 * Revision 1.1.1.1  1995/05/19 18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.10  1994/10/12  19:43:45  hjp
 * added nstrend.
 * replaced nstrncpy by nstrlimcpy.
 *
 * Revision 1.9  1994/04/22  01:15:47  hjp
 * fixed prototype of memmem
 *
 * Revision 1.8  1992/12/17  00:35:20  hjp
 * added EOF, parsechar
 *
 * Revision 1.7  1992/12/15  23:29:56  hjp
 * added nstrlimcpy and nstrpar
 *
 * Revision 1.6  1992/05/30  23:16:47  hjp
 * included <string.h>, used _ macro for memmem prototype
 *
 * Revision 1.5  1992/05/30  23:04:14  hjp
 * added memmem
 *
 * Revision 1.4  1992/05/25  19:30:35  hjp
 * added nstr[n]cpy
 *
 * Revision 1.3  1992/05/24  12:59:00  hjp
 * fixed embarrassing error with declaring strdup
 *
 * Revision 1.2  1992/05/24  12:45:59  hjp
 * added strdup
 *
 * Revision 1.1  1992/05/24  12:25:32  hjp
 * Initial revision
 *
 */
#ifndef I_ant_string_h
#define I_ant_string_h

#include <string.h>

#if __STDC__
#	define _(x)	x
#else
#	define _(x)	()
#endif

#ifndef EOF
#	define	EOF	(-1)
#endif

#define STREQ(a,b)	(strcmp((a),(b)) == 0)


/*
 * Function:	strdup 
 * Purpose:	duplicate string on the heap and return pointer to it.
 * In:		s	String to be duplicated
 * Returns:	Pointer to duplicated string or NULL, if not enough 
 *		space for new string.
 */
extern char *strdup _((const char *s));


/*
 * Function:	nstrcpy
 * Purpose:	copy string
 * In:		src ... zero terminated string to be copied
 * Out:		dst ...	Pointer to destination area.
 * Returns:	Pointer to the terminating NUL character of dst.
 */
extern char *nstrcpy _((char *dst, const char *src));


/*
 * Function:	nstrlimcpy
 * Purpose:	copy string into buffer of limited size. The resulting 
 *		string at dst is always zero-terminated and at most
 *		limit - dst bytes long (incl. NUL).
 * In:		src ...		zero terminated string to be copied
 * In:		limit ...	pointer to the first byte after the
 *				buffer.
 * Out:		dst ...		Pointer to destination area.
 * Returns:	Pointer to the terminating NUL character of dst.
 *
 * NOTE:	The name nstrncpy is retained to maintain compatibility
 *		with an older version of this library. Don't use!
 */
extern char *nstrlimcpy _((char *dst, const char *src, const char *limit));
#define nstrncpy nstrlimcpy


/*
 * Function:	memmem
 * Purpose:	search for first occurrence of wanted in mem.
 * In:		mem	... pointer to memory area to be searched
 *		memsize ... size of    -"-
 *		wanted	... pointer to search pattern
 *		wantedsize ... size of -"-
 * Returns:	Pointer to first occurence or NULL if no match was
 *		found.
 */
void *memmem _((const void *mem, size_t memsize, const void *wanted, size_t wantedsize));


/*
 *	nstrpar
 *
 *	decompose string into parameters.
 *
 *	Parameters are strings delimited by white space or C-like strings.
 *
 *	NOTES:
 *		The closing quote of a quoted parameter is discarded.
 *		A string of the form ``"abc"def'' is decomposed into two
 *		parameters: ``"abc'' and ``def''.
 *
 *	BUGS:
 *		A string ending in \, where the last parameter starts with
 *		a quote will cause garbage to be returned as the last
 *		parameter and may corrupt data following string.
 */
char *nstrpar _((char *string));


/*
 * Function:	parsechar
 * Purpose:	return character from a string in C syntax and advance pointer
 *		to next character.
 * InOut:	pp  ... Pointer to Pointer to current character.
 * Returns:	The current character, or EOF if the end of the string
 *		('"', '\n' or '\0') is encountered.
 */
int parsechar _((char ** pp));


/*
 * Function:	nstrend
 * Purpose:	return end of string
 * In:		s ... zero terminated string
 * Returns:	Pointer to the terminating NUL character of s.
 */
extern char *nstrend _((const char *s));


#endif
