#include "time.h"

#define USE_CLOCK		1
#define USE_GETTIMEOFDAY	2
#define USE_TIMES		3
#define USE_TIME		4

#if __MSDOS__
   /* MSDOS doesn't do per process accounting, so real and cpu time are
    * the same.
    */
# if !REAL_METHOD
#  define REAL_METHOD	USE_CLOCK
# endif
# if !CPU_METHOD
#  define CPU_METHOD	USE_CLOCK
# endif
#endif

#if __ultrix || __osf || __linux
   /* On Ultrix and OSF/1 times doesn't deliver the full resolution of 
    * the system clock, so we use gettimeofday for real time and
    * clock for cpu time.
    */
# if !REAL_METHOD
#  define REAL_METHOD	USE_GETTIMEOFDAY
# endif
# if !CPU_METHOD
#  define CPU_METHOD	USE_CLOCK
# endif
#endif


#if __unix || __minix
   /* times is available on just about any unix system, and delivers
    * both real and cpu time.
    */
# if !REAL_METHOD
#  define REAL_METHOD	USE_TIMES
# endif
# if !CPU_METHOD
#  define CPU_METHOD	USE_TIMES
# endif
#endif


#if !REAL_METHOD
# if __MSDOS__
   /* MSDOS doesn't do per process accounting, so real and cpu time are
    * the same.
    */
#  define REAL_METHOD	USE_CLOCK
# elif __ultrix || __osf
   /* On Ultrix and OSF/1 times doesn't deliver the full resolution of 
    * the system clock.
    */
#  define REAL_METHOD	USE_GETTIMEOFDAY
# elif __unix || __minix
   /* times is available on just about any unix system, and delivers
    * both real and cpu time.
    */
#  define REAL_METHOD	USE_TIMES
# else
   /* time should be available on any C system, but there are no 
    * guarantees about its resolution. All systems I know have a
    * resolution of 1 second, which isn't that great.
    */
#  define REAL_METHOD	USE_TIME
# endif
#endif


#if !CPU_METHOD
# if __unix && !(__osf || __ultrix) || __minix
   /* On systems where we use times to get the real time we can use
    * it for CPU time, too and save one system call.
    */
#  define CPU_METHOD	USE_TIMES
# else
   /* clock should work everywhere
    */
#  define CPU_METHOD	USE_CLOCK
# endif
#endif


#include <time.h>

#if REAL_METHOD == USE_TIMES || CPU_METHOD == USE_TIMES
#include <sys/times.h>
#include <unistd.h>
#endif

#if REAL_METHOD == USE_GETTIMEOFDAY
#include <sys/time.h>
#include <unistd.h>
#endif

static double start_real;
static double start_cpu;

#if REAL_METHOD == USE_CLOCK || CPU_METHOD == USE_CLOCK
	static double clock_res;
#endif
#if REAL_METHOD == USE_TIMES || CPU_METHOD == USE_TIMES
	static double times_res;
	static struct tms tms;
#endif
#if REAL_METHOD == USE_TIME
	static time_t time_ref;
#endif

void resettimer (void) {
	#if REAL_METHOD == USE_CLOCK
		if (!clock_res) {
			clock_res = 1.0 / CLOCKS_PER_SEC;
		}
		start_real = (double) clock () * clock_res;
	#endif
	#if REAL_METHOD == USE_TIMES
		if (!times_res) {
			times_res = 1.0 / sysconf (_SC_CLK_TCK);
		}
		start_real = times (&tms) * times_res;
	#endif
	#if REAL_METHOD == USE_GETTIMEOFDAY
		struct timeval tv;
		struct timezone tz;

		gettimeofday (&tv, &tz);
		start_real = tv.tv_sec + tv.tv_usec * 1E-6;
	#endif
	#if REAL_METHOD == USE_TIME
		time_ref = time (NULL);
	#endif
	#if CPU_METHOD == USE_TIMES
		#if CPU_METHOD != REAL_METHOD
			if (!times_res) {
				times_res = 1.0 / sysconf (_SC_CLK_TCK);
			}
			times (&tms);
		#endif
		start_cpu = (tms.tms_utime + tms.tms_stime) * times_res;
	#endif
	#if CPU_METHOD == USE_CLOCK
		#if CPU_METHOD == REAL_METHOD
			start_cpu = start_real;
		#else
			if (!clock_res) {
				clock_res = 1.0 / CLOCKS_PER_SEC;
			}
			start_cpu = clock () * clock_res;
		#endif
	#endif
}
void gettimer (double *real, double *cpu) {
	#if REAL_METHOD == USE_CLOCK
		*real = (double) clock () * clock_res - start_real;
	#endif
	#if REAL_METHOD == USE_TIMES
		*real = times (&tms) * times_res - start_real;
	#endif
	#if REAL_METHOD == USE_GETTIMEOFDAY
		struct timeval tv;
		struct timezone tz;

		gettimeofday (&tv, &tz);
		*real = tv.tv_sec + tv.tv_usec * 1E-6 - start_real;
	#endif
	#if REAL_METHOD == USE_TIME
		time_t now;
		*real = difftime (time (NULL), time_ref)
	#endif
	#if CPU_METHOD == USE_TIMES
		#if CPU_METHOD != REAL_METHOD
			times (&tms);
		#endif
		*cpu = (tms.tms_utime + tms.tms_stime) * times_res - start_cpu;
	#endif
	#if CPU_METHOD == USE_CLOCK
		#if CPU_METHOD == REAL_METHOD
			*cpu = *real;
		#else
			*cpu = clock () * clock_res - start_cpu;
		#endif
	#endif
}
