#if TEST
#include <stdio.h>
#endif

#include <string.h>
#include <ant/alloc.h>

#include <ant/url.h>

urlT *new_urlT(void) {
    urlT *pu = emalloc(sizeof(*pu));
    pu->scheme = NULL;
    pu->location = NULL;
    pu->path = NULL;
    pu->params = NULL;
    pu->query = NULL;
    pu->fragment = NULL;
    return pu;
}


urlT *parse_url(const char *url) {
    /* split an (absolute or relative) URL into its components
     * as described in RFC 1808
     */
    char *p;
    char *sourl;
    char *eourl;
    size_t schemelen;
    urlT *pu = new_urlT();

    /* 2.4.1: parse fragment identifier */
    if ((p = strchr(url, '#'))) {
        eourl = p;
        pu->fragment = strdup(eourl+1);
    } else {
        eourl = (char *)url + strlen(url);
    }
    /* 2.4.2: parse scheme */
    schemelen = strspn(url, "ABCDEFGHIIJKLMNOPQRSTUVWXYZ"
                            "abcdefghiijklmnopqrstuvwxyz"
                            ".+-");
    if (url[schemelen] == ':') {
        p = emalloc(schemelen+1);
        memcpy(p, url, schemelen);
        p[schemelen] = '\0';
        pu->scheme = p;
        schemelen++;
    } else {
        schemelen = 0;
    }
    sourl = (char *)url + schemelen;
    /* 2.4.3: parse network location/login */
    if (strncmp (sourl, "//", 2) == 0) {
        char *p2;

        if (!(p = strchr(sourl + 2, '/'))) {
            p = eourl;
        }

        p2 = emalloc(p-(sourl+2)+1);
        memcpy(p2, sourl+2, p-(sourl+2));
        p2[p-(sourl+2)] = '\0';
        sourl = p;
        pu->location = p2;
    } 
    p = emalloc(eourl-sourl+1);
    memcpy(p, sourl, eourl-sourl);
    p[eourl-sourl] = '\0';
    eourl = p + (eourl - sourl);
    sourl = p;
    /* 2.4.4: remove query information */
    if ((p = strchr(sourl, '?'))) {
        *p = '\0';
        pu->query = strdup(p+1);
    }
    /* 2.4.5: remove parameters */
    if ((p = strchr(sourl, ';'))) {
        *p = '\0';
        pu->params = strdup(p+1);
    }
    pu->path=sourl;
    return pu;
}


void free_urlT(urlT *pu) {
    free(pu->scheme); pu->scheme = NULL;
    free(pu->location); pu->location = NULL;
    free(pu->path); pu->path = NULL;
    free(pu->params); pu->params = NULL;
    free(pu->query); pu->query = NULL;
    free(pu->fragment); pu->fragment = NULL;
    free(pu);
}


char *combine_urls(const char *baseurl, const char *emburl) {
    /* combine two URLs
     * as described in RFC 1808, Section 4
     */
    
    urlT *bu = NULL, *eu = NULL, *cu = NULL;
    char *scu;
    size_t sz = 0;
    size_t off = 0;
    char *p1 = NULL;
    typedef struct pathsegT pathsegT;
    struct pathsegT {
        char *s;
        pathsegT *next;
        pathsegT *prev;
    };
    pathsegT *cul = NULL;  /* Combined URL as list of pathsegs */
    pathsegT *psp = NULL;
    pathsegT *last = NULL;

    /* Step 2 */
    if (strcmp(emburl, "") == 0) {
        scu = (char *)baseurl;
        goto done;
    }
    bu = parse_url(baseurl);
    eu = parse_url(emburl);

    if (eu->scheme) {
        scu = (char *)emburl;
        goto done;
    }
    cu = new_urlT();
    cu->scheme = strdup(bu->scheme);

    /* Step 3 */
    if(!eu->location) {
        cu->location = strdup(bu->location);
        
        /* Step 4 */
        if (eu->path[0] != '/') {
            /* Step 5 */
            if (strcmp (eu->path, "") == 0) {
                cu->path = strdup(bu->path);
                if (!eu->params) {
                    cu->params = strdup(bu->params);
                    if (!eu->query) {
                        cu->query = strdup(bu->query);
                    }
                }
            } else {
                /* Step 6 */
                char *p1, *p2;
                int stop = 0;

                for (p1 = bu->path;
                     (p2 = strchr(p1, '/')); 
                     p1 = p2 + 1) {
                    if (psp) {
                        psp->next = emalloc(sizeof(*psp->next));
                        psp->next->prev = psp;
                        psp = psp->next;
                    } else {
                        psp = emalloc(sizeof(*psp->next));
                        cul = psp;
                        cul->prev = NULL;
                    }
                    psp->s = emalloc(p2 - p1 + 2);
                    memcpy(psp->s, p1, p2 - p1 + 1);
                    psp->s[p2 - p1 + 1] = '\0';
                    psp->next = NULL;
                    last = psp;
                }
                /* The following loop is a little bit tricky:
                 * Basically, when we don't find a / we look for the
                 * end of the string and set stop so that we will
                 * exit before the next iteration
                 */
                for (p1 = eu->path;
                     !stop &&
                     ((p2 = strchr(p1, '/')) ||
                      ((p2 = strchr(p1, '\0')), stop = 1)); 
                     p1 = p2 + 1) {
                    if (psp) {
                        psp->next = emalloc(sizeof(*psp->next));
                        psp->next->prev = psp;
                        psp = psp->next;
                    } else {
                        psp = emalloc(sizeof(*psp->next));
                        cul = psp;
                        cul->prev = NULL;
                    }
                    psp->s = emalloc(p2 - p1 + 1 + 1);
                    memcpy(psp->s, p1, p2 - p1 + 1);
                    psp->s[p2 - p1 + 1] = '\0';
                    psp->next = NULL;
                    last = psp;
                }

                /* Step 6 a */
                for (psp = cul;
                     psp;
                     ) {
                    if (strcmp(psp->s, "./") == 0) {
                        pathsegT *next;
                        if (psp->prev) {
                            psp->prev->next = psp->next;
                        } else {
                            cul = psp->next;
                        }
                        if (psp->next) {
                            psp->next->prev = psp->prev;
                        } else {
                            last = psp->prev;
                        }

                        next = psp->next;
                        psp->next = psp->prev = NULL;
                        free(psp->s);
                        free(psp);
                        psp = next;
                    } else {
                        psp = psp->next;
                    }
                }

                /* Step 6 b */
                if (strcmp (last->s, ".") == 0) {
                    last->next = NULL;
                    free(last->s);
                    last->s = NULL;
                    last = last->prev;
                    last->next->prev = NULL;
                    free(last->next);
                    last->next = NULL;
                }

                /* Step 6 c */
                for (psp = cul;
                     psp;
                     ){
                    if (strcmp(psp->s, "../") == 0 &&
                        psp->prev &&
                        (psp->prev != cul || strcmp(psp->prev->s, "/")) &&
                        strcmp(psp->prev->s, "../")) {
                        pathsegT *next;
                        pathsegT *prev;
                        if (psp->prev->prev) {
                            psp->prev->prev->next = psp->next;
                        } else {
                            cul = psp->next;
                        }
                        if (psp->next) {
                            psp->next->prev = psp->prev->prev;
                        } else {
                            last = psp->prev;
                        }

                        next = psp->next;
                        prev = psp->prev;
                        psp->next = psp->prev = NULL;
                        free(psp->s);
                        free(psp);
                        prev->next = prev->prev = NULL;
                        free(prev->s);
                        free(prev);
                        psp = next;
                    } else {
                        psp = psp->next;
                    }
                }

                /* Step 6 d */
                psp = last;
                if (strcmp(psp->s, "..") == 0 && psp->prev) {
                    pathsegT *next;
                    pathsegT *prev;
                    if (psp->prev->prev) {
                        psp->prev->prev->next = psp->next;
                    } else {
                        cul = psp->next;
                    }
                    if (psp->next) {
                        psp->next->prev = psp->prev->prev;
                    } else {
                        last = psp->prev;
                    }

                    next = psp->next;
                    prev = psp->prev;
                    psp->next = psp->prev = NULL;
                    free(psp->s);
                    free(psp);
                    prev->next = prev->prev = NULL;
                    free(prev->s);
                    free(prev);
                    psp = next;
                }

                p1 = NULL;
                sz = 0;
                off = 0;
                for (psp = cul; psp; psp = psp->next) {
                    size_t ln = strlen(psp->s);
                    if (off + ln > sz) {
                        sz = sz * 2 + 20;
                        p1 = erealloc(p1, sz);
                    }
                    memcpy(p1 + off, psp->s, ln);
                    off += ln;
                }
                if (off + 1 > sz) {
                    sz = sz * 2 + 20;
                    p1 = erealloc(p1, sz);
                }
                p1[off] = '\0';
                cu->path = p1;
            }
        } else {
            cu->path = strdup(eu->path);
        }
    } else {
        cu->location = strdup(eu->location);
    }
    /* Step 7 */
    if (!cu->params && eu->params) {
        cu->params = strdup(eu->params);
    }
    if (!cu->query && eu->query) {
        cu->query = strdup(eu->query);
    }
    if (!cu->fragment && eu->fragment) {
        cu->fragment = strdup(eu->fragment);
    }
    p1 = NULL;
    sz = 0;
    off = 0;
    if (cu->scheme) {
        size_t ln = strlen(cu->scheme);
        while (ln + 1 + 1 > sz) {
            sz = sz * 2 + 20;
            p1 = erealloc(p1, sz);
        }
        memcpy(p1, cu->scheme, ln);
        off = ln;
        p1[off++] = ':';
    }
    if (cu->location) {
        size_t ln = strlen(cu->location);
        if (off + 2 + ln > sz) {
            sz = sz * 2 + 20;
            p1 = erealloc(p1, sz);
        }
        p1[off++] = '/';
        p1[off++] = '/';
        memcpy(p1 + off, cu->location, ln);
        off += ln;
    }
    if (cu->path) {
        size_t ln = strlen(cu->path);
        if (off + ln > sz) {
            sz = sz * 2 + 20;
            p1 = erealloc(p1, sz);
        }
        memcpy(p1 + off, cu->path, ln);
        off += ln;
    }
    if (cu->params) {
        size_t ln = strlen(cu->params);
        if (off + 1 + ln > sz) {
            sz = sz * 2 + 20;
            p1 = erealloc(p1, sz);
        }
        p1[off++] = ';';
        memcpy(p1 + off, cu->params, ln);
        off += ln;
    }
    if (cu->query) {
        size_t ln = strlen(cu->query);
        if (off + 1 + ln > sz) {
            sz = sz * 2 + 20;
            p1 = erealloc(p1, sz);
        }
        p1[off++] = '?';
        memcpy(p1 + off, cu->query, ln);
        off += ln;
    }
    if (cu->fragment) {
        size_t ln = strlen(cu->fragment);
        if (off + 1 + ln > sz) {
            sz = sz * 2 + 20;
            p1 = erealloc(p1, sz);
        }
        p1[off++] = '#';
        memcpy(p1 + off, cu->fragment, ln);
        off += ln;
    }
    if (off + 1 > sz) {
        sz = sz * 2 + 20;
        p1 = erealloc(p1, sz);
    }
    p1[off] = '\0';
    scu = p1;

done:
    if (bu) free_urlT(bu);
    if (eu) free_urlT(eu);
    if (cu) free_urlT(cu);
    if (cul) {
        for (psp = cul; psp; ) {
            pathsegT *next = psp->next;
            psp->next = psp->prev = NULL;
            free(psp->s);
            free(psp);
            psp = next;
        }
    }
    return scu;
}


#if TEST
char *testcases[][3] = {
    /* These don't work yet */
    { "http://a/b/c/d;p?q#f", "../../../g",	"http://a/../g"		},
    { "http://a/b/c/d;p?q#f", "../../../../g",	"http://a/../../g"		},

    /* From RFC 1808, 5.1 Normal examples */
    { "http://a/b/c/d;p?q#f", "g:h",        	"g:h" },
    { "http://a/b/c/d;p?q#f", "g",          	"http://a/b/c/g" },
    { "http://a/b/c/d;p?q#f", "./g",        	"http://a/b/c/g" },
    { "http://a/b/c/d;p?q#f", "g/",         	"http://a/b/c/g/" },
    { "http://a/b/c/d;p?q#f", "/g",         	"http://a/g"	},
    { "http://a/b/c/d;p?q#f", "//g",		"http://g"	},
    { "http://a/b/c/d;p?q#f", "?y",		"http://a/b/c/d;p?y"	},
    { "http://a/b/c/d;p?q#f", "g?y",		"http://a/b/c/g?y"	},
    { "http://a/b/c/d;p?q#f", "g?y/./x",	"http://a/b/c/g?y/./x"	},
    { "http://a/b/c/d;p?q#f", "#s",		"http://a/b/c/d;p?q#s"	},
    { "http://a/b/c/d;p?q#f", "g#s",		"http://a/b/c/g#s"	},
    { "http://a/b/c/d;p?q#f", "g#s/./x",	"http://a/b/c/g#s/./x"	},
    { "http://a/b/c/d;p?q#f", "g?y#s",		"http://a/b/c/g?y#s"	},
    { "http://a/b/c/d;p?q#f", ";x",		"http://a/b/c/d;x"	},
    { "http://a/b/c/d;p?q#f", "g;x",		"http://a/b/c/g;x"	},
    { "http://a/b/c/d;p?q#f", "g;x?y#s",	"http://a/b/c/g;x?y#s"	},
    { "http://a/b/c/d;p?q#f", ".",		"http://a/b/c/"		},
    { "http://a/b/c/d;p?q#f", "./",		"http://a/b/c/"		},
    { "http://a/b/c/d;p?q#f", "..",		"http://a/b/"		},
    { "http://a/b/c/d;p?q#f", "../",		"http://a/b/"		},
    { "http://a/b/c/d;p?q#f", "../g",		"http://a/b/g"		},
    { "http://a/b/c/d;p?q#f", "../..",		"http://a/"		},
    { "http://a/b/c/d;p?q#f", "../../",		"http://a/"		},
    { "http://a/b/c/d;p?q#f", "../../g",	"http://a/g"		},

    /* From RFC 1808, 5.2 Abnormal examples */
    { "http://a/b/c/d;p?q#f", "",		"http://a/b/c/d;p?q#f"		},
    { "http://a/b/c/d;p?q#f", "../../../g",	"http://a/../g"		},
    { "http://a/b/c/d;p?q#f", "../../../../g",	"http://a/../../g"		},
    { "http://a/b/c/d;p?q#f", "/./g",		"http://a/./g"		},
    { "http://a/b/c/d;p?q#f", "/../g",		"http://a/../g"		},
    { "http://a/b/c/d;p?q#f", "g.",		"http://a/b/c/g."		},
    { "http://a/b/c/d;p?q#f", ".g",		"http://a/b/c/.g"		},
    { "http://a/b/c/d;p?q#f", "g..",		"http://a/b/c/g.."		},
    { "http://a/b/c/d;p?q#f", "..g",		"http://a/b/c/..g"		},
    { "http://a/b/c/d;p?q#f", "./../g",		"http://a/b/g"		},
    { "http://a/b/c/d;p?q#f", "./g/.",		"http://a/b/c/g/"		},
    { "http://a/b/c/d;p?q#f", "g/./h",		"http://a/b/c/g/h"		},
    { "http://a/b/c/d;p?q#f", "g/../h",		"http://a/b/c/h"		},
    { "http://a/b/c/d;p?q#f", "http:g",		"http:g"		},
    { "http://a/b/c/d;p?q#f", "http:",		"http:"		},
    { NULL }
};


int main(int argc, char **argv) {
    char *result;
    int rc = 0;
    char *(*p)[3];

    for (p = testcases; **p; p++) {
        printf("%s + %s == %s:", (*p)[0], (*p)[1], (*p)[2]);
        fflush(stdout);
        result = combine_urls((*p)[0], (*p)[1]);
	if (strcmp(result, (*p)[2]) == 0) {
            printf(" Ok\n");
	} else {
	    printf(" != %s\n", result);
	    rc = 1;
	}
    }
    return rc;
}
#endif
