char _wrminixhdr_c_rcs_id [] =
	"$Id: wrminixhdr.c,v 1.2 1995/05/19 19:35:01 hjp Exp $";
/* Write the header of a Minix executable
 *
 * $Log: wrminixhdr.c,v $
 * Revision 1.2  1995/05/19 19:35:01  hjp
 * Quick'n'dirty port to linux.
 * Compiles but no guarantees that it works.
 *
 * Revision 1.1.1.1  1995/05/19  18:42:31  hjp
 * Imported from Minix. We'll see where main development will be ...
 *
 * Revision 1.1  1993/06/28  22:54:44  hjp
 * Initial revision
 *
 */
#include <assert.h>
#include <errno.h>
#include <string.h>

#include <a.out.h>

#include <ant/globals.h>
#include <ant/io.h>
#include <ant/readheader.h>

#define eputc(c, out) if (putc (c, out) == EOF) return EOF
#define ewritelong(c, out) if (writelong (c, out) == EOF) return EOF

int write_minix_header (
	exec_headerT *hdr,	/* In: header structure to be written */
	FILE *out		/* In: File to write header to. */
)
/* Write a minix ack executable header to a file.
 * Returns: 0 if ok, -1, if some error occured. errno may or may not
 * 	be set to a meaningful value.
 */
{
#if __minix
	int flags = 0, cpu = 0, hdrlen = 32;
	long syms = 0;	/* no symbol table (yet) */
	int (*writelong) (unsigned long, FILE *);

	switch (hdr->type) {
	case MINIX_I8086C_EXEC:
		cpu = A_I8086;
		writelong = writeulongle;
		break;
	case MINIX_I8086S_EXEC:
		cpu = A_I8086;
		writelong = writeulongle;
		flags |= A_SEP;
		break;
	case MINIX_I80386C_EXEC:
		cpu = A_I80386;
		writelong = writeulongle;
		break;
	case MINIX_I80386S_EXEC:
		cpu = A_I80386;
		writelong = writeulongle;
		flags |= A_SEP;
		break;
	case MINIX_M68000C_EXEC:
		cpu = A_M68K;
		writelong = writeulongbe;
		break;
	case MINIX_M68000S_EXEC:
		cpu = A_M68K;
		writelong = writeulongbe;
		flags |= A_SEP;
		break;
	case GNU_BE_PAGE:
	case GNU_BE_PURE:
	case GNU_BE_OBJ:
		cpu = A_M68K;	/* Guess */
		writelong = writeulongbe;
		flags |= A_SEP;
		break;
	case GNU_LE_PAGE:
	case GNU_LE_PURE:
	case GNU_LE_OBJ:
		cpu = A_I80386;
		switch (hdr->entry) {
		case 0:
			break;
		case 0x1000:
			flags |= A_UZP;
			break;
		default:
			eprintf ("%s.write_minix_header: (warning) unexpected entry point %#lx\n",
				 cmnd, hdr->entry);
		}
		writelong = writeulongle;
		break;
	default:
		assert (0);
	}

	eputc (MAGIC_MINIX0, out);
	eputc (MAGIC_MINIX1, out);
	eputc (flags, out);
	eputc (cpu, out);
	eputc (hdrlen, out);
	eputc (0, out);	/* reserved */
	eputc (0, out);	/* version stamp */
	eputc (0, out);
	ewritelong (hdr->text, out);
	ewritelong (hdr->data, out);
	ewritelong (hdr->bss, out);
	ewritelong (hdr->entry, out);
	if (flags & A_SEP) {
		ewritelong (hdr->total - hdr->text, out);
	} else {
		ewritelong (hdr->total, out);
	}
	ewritelong (syms, out);
	return 0;
#else
	/* other systems don't know about Minix a.out format
	 */
	return EOF;
#endif
}


void ewrite_minix_header (exec_headerT *hdr, FILE *out)
/* Do the same thing as write_minix_header, but dump terminate 
 * program if something goes wrong.
 *
 * Uses:	dump_on_exit	If true, dump core on error
 */
{
	if (write_minix_header (hdr, out) == EOF) {
		eprintf ("%s: cannot write minix header: %s\n",
			 cmnd, strerror (errno));
		if (dump_on_exit) abort (); else exit ();
	}
}
